@@ -50,7 +50,7 @@ try:
 
     import asyncio, platform, psutil, socket, json, re
     import signal, shutil, termios, tty, select, fcntl
-    import subprocess
+    import subprocess, shlex
     from collections import Counter
     from pathlib import Path
     from datetime import datetime, timedelta
@@ -59,12 +59,15 @@ except KeyboardInterrupt:
 
     os._exit(0)
 
+
 def exit_on_buffer():
     if select.select([stdin], [], [], 0)[0] != []:
         os._exit(0)
 
+
 exit_on_buffer()
 
+
 def terminal_size() -> tuple:
     try:
         size = shutil.get_terminal_size(fallback=(999, 999))
@@ -89,6 +92,8 @@ accent_dir = 1
 _nansi = {}
 _nansi_order = []
 _last_run_data = {}
+_run = []
+_shell = os.environ["SHELL"]
 
 
 def once(func):
@@ -642,6 +647,7 @@ async def count_failed_systemd() -> dict:
     system_statuses = await get_service_statuses(
         "systemctl list-units --type=service --no-legend --no-pager | awk '{print $4}'"
     )
+
     # user_statuses = await get_service_statuses(
     #     "systemctl --user list-units --type=service --no-legend --no-pager | awk '{print $4}'"
     # )
@@ -775,7 +781,9 @@ def nansi(inpt: str) -> str:
 def animation() -> str:
     global awidth, tix, accent_dir
     prompt = (
-        "Press any key to enter -- System Ready -- " if os.geteuid() else "CAUTION -!!- "
+        "Press any key to enter -- System Ready -- "
+        if os.geteuid()
+        else "CAUTION -!!- "
     )
     repeated = prompt * (((awidth - 4) // len(prompt)) + 3)
 
@@ -959,14 +967,21 @@ async def main() -> None:
         if upd_str:
             msg.append(upd_str + "\n")
 
-    if not hush_news:
-        if hush_updates:
-            msg.append("\n")
-        if isinstance(updates, list):
-            news = updates[2]
-            msg += [(news if news else "Failed to fetch news.\n"), "\n"]
-        else:
-            msg += ["Failed to fetch news.", "\n", "\n"]
+    if os.getlogin() == "bred" and os.path.exists("/usr/bin/Bakery"):
+        msg.append(f"{colors.yellow_t}Setup is {colors.bold}INCOMPLETE{colors.endc}!\n")
+        msg.append(
+            f"If you wish to complete it from the command line, run `{colors.bland_t}{colors.bold}Bakery --tui{colors.endc}`\n"
+        )
+        msg.append("\n")
+    else:
+        if not hush_news:
+            if hush_updates:
+                msg.append("\n")
+            if isinstance(updates, list):
+                news = updates[2]
+                msg += [(news if news else "Failed to fetch news.\n"), "\n"]
+            else:
+                msg += ["Failed to fetch news.", "\n", "\n"]
 
     show_url = False
     if not hush_smart:
@@ -1033,10 +1048,10 @@ async def main() -> None:
             msg[0] = msg[0][:kernst] + colors.warning + smallmsg + colors.bland_t + ")"
         else:
             stm = ((sps - len(smallmsg)) // 2) + 1
-            kernl.insert(kernst+1, colors.warning)
+            kernl.insert(kernst + 1, colors.warning)
             kernl.insert(kernend, colors.bland_t)
             for i in range(len(smallmsg)):
-                kernl[kernst+stm+i] = smallmsg[i]
+                kernl[kernst + stm + i] = smallmsg[i]
             msg[0] = "".join(kernl)
     msg.append(animation())
 
@@ -1057,6 +1072,36 @@ async def suspend(until: float) -> None:
         await delay(Time_Tick)
 
 
+def shell_inject(text: str) -> bool:
+    try:
+        for ch in text:
+            fcntl.ioctl(stdin, termios.TIOCSTI, ch.encode())
+        return True
+    except KeyboardInterrupt:
+        pass
+    except:
+        pass
+    return False
+
+
+def shortcut_handler(text: str) -> bool:
+    global _run
+    if text and text[0] in shortcuts.keys():
+        shortcut = shortcuts[text[0]]
+        if not shell_inject(f" {shortcut}\n"):
+            _run = [_shell, "-c", f'eval "{shortcut}"; exec {_shell}']
+    else:
+        shell_inject(text)
+
+
+def kill_parent(sig=signal.SIGKILL):
+    parent_pid = os.getppid()
+    try:
+        os.kill(parent_pid, sig)
+    except:
+        pass
+
+
 async def loop_main() -> None:
     global screensaver_mode
     # Pure aneurism.
@@ -1083,6 +1128,16 @@ async def loop_main() -> None:
             pass
         except:
             pass
+
+        if _run:
+            try:
+                subprocess.run(_run)
+                kill_parent()
+            except KeyboardInterrupt:
+                pass
+            except:
+                pass
+
         os._exit(0)
 
     signal.signal(signal.SIGINT, handle_exit)
@@ -1098,15 +1153,16 @@ async def loop_main() -> None:
                 if dr != []:
                     buf = os.read(fd, 4096).decode(errors="ignore")
                     if (
-                        buf.isalnum() or len(buf) - 1 or ord(buf) in [4, 12]
+                        buf.isalnum()
+                        or len(buf) - 1
+                        or ord(buf) in [4, 12]
+                        or buf[0] in shortcuts.keys()
                     ) and not screensaver_mode:
                         # Do not inject if not a alphanum / Ctrl-D / Arrow key
-                        try:
-                            for ch in buf:
-                                fcntl.ioctl(stdin, termios.TIOCSTI, ch.encode())
-                        except:  # Injection failed, just exit.
-                            pass
+                        shortcut_handler(buf)
                     handle_exit()
+            if Onetime:
+                handle_exit()
             await suspend(stamp + Time_Refresh)
     except Exception as err:
         print("\nUNHANDLED EXCEPTION!\n")
@@ -1128,7 +1184,9 @@ Hush_Disks = None
 Hush_Smart = None
 Time_Tick = 0.1
 Time_Refresh = 0.25
+Onetime = False
 
+shortcuts = {}
 
 newsrc_path = os.path.expanduser("~/.newsrc")
 if os.path.isfile(newsrc_path):
@@ -1154,7 +1212,14 @@ else:  # Install default template
                 + "# Hush_Disks = False\n"
                 + "# Hush_Smart = False\n"
                 + "# Time_Tick = 0.1\n"
-                + "# Time_Refresh = 0.25\n\n"
+                + "# Time_Refresh = 0.25\n"
+                + "# Onetime = False\n"
+                + "\n"
+                + "# Shortcuts configuration\n"
+                + "\n"
+                + "# shortcuts = {\n"
+                + '#     "1": "bredos-config",\n'
+                + "# }\n"
             )
     except:
         pass
